@@ -1,7 +1,7 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
@@ -1168,30 +1168,26 @@ void update_host_status(int status, host_t *host, ping_t *ping, int availability
 				if (set.ping_failure_count == 1) {
 					snprintf(host->status_fail_date, 40, "%s", current_date);
 				}
-
-			/* host is down, but not ready to issue log message */
 			} else {
 				/* host down for the first time, set event date */
 				if (host->status_event_count == 1) {
 					snprintf(host->status_fail_date, 40, "%s", current_date);
 				}
 			}
-
-		/* host is recovering, put back in failed state */
 		} else if (host->status == HOST_RECOVERING) {
+			/* host is recovering, put back in failed state */
 			host->status_event_count = 1;
 			host->status = HOST_DOWN;
-
-		/* host was unknown and now is down */
 		} else if (host->status == HOST_UNKNOWN) {
+			/* host was unknown and now is down */
 			host->status = HOST_DOWN;
 			host->status_event_count = 0;
 		} else {
 			host->status_event_count++;
 		}
-
-	/* host is up!! */
 	} else {
+		/* host is up!! */
+
 		/* update total polls and availability */
 		host->total_polls = host->total_polls + 1;
 		host->availability = hundred_percent * (host->total_polls - host->failed_polls) / host->total_polls;
@@ -1232,7 +1228,7 @@ void update_host_status(int status, host_t *host, ping_t *ping, int availability
 			* host->avg_time) + ping_time) / (host->total_polls-host->failed_polls);
 
 		/* the host was down, now it's recovering */
-		if ((host->status == HOST_DOWN) || (host->status == HOST_RECOVERING )) {
+		if ((host->status == HOST_DOWN) || (host->status == HOST_RECOVERING)) {
 			/* just up, change to recovering */
 			if (host->status == HOST_DOWN) {
 				host->status = HOST_RECOVERING;
@@ -1255,19 +1251,23 @@ void update_host_status(int status, host_t *host, ping_t *ping, int availability
 
 				/* reset the event counter */
 				host->status_event_count = 0;
-			/* host is recovering, but not ready to issue log message */
 			} else {
 				/* host recovering for the first time, set event date */
 				if (host->status_event_count == 1) {
 					snprintf(host->status_rec_date, 40, "%s", current_date);
 				}
 			}
+		} else if (host->status_event_count > 0) {
+			/* host was unknown and now is up */
+			host->status = HOST_UP;
+			host->status_event_count = 0;
 		} else {
-		/* host was unknown and now is up */
+			/* host was unknown and now is up */
 			host->status = HOST_UP;
 			host->status_event_count = 0;
 		}
 	}
+
 	/* if the user wants a flood of information then flood them */
 	if (set.log_level >= POLLER_VERBOSITY_HIGH) {
 		if ((host->status == HOST_UP) || (host->status == HOST_RECOVERING)) {
