@@ -1,12 +1,12 @@
 /*
  ex: set tabstop=4 shiftwidth=4 autoindent:
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2023 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU Lesser General Public              |
  | License as published by the Free Software Foundation; either            |
- | version 2.1 of the License, or (at your option) any later version. 	|
+ | version 2.1 of the License, or (at your option) any later version.      |
  |                                                                         |
  | This program is distributed in the hope that it will be useful,         |
  | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
@@ -150,6 +150,7 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 	char query10[BUFSIZE];
 	char query11[BUFSIZE];
 	char *query12 = NULL;
+	char query13[BUFSIZE];
 	char posuffix[BUFSIZE];
 
 	int query1_len   = 0;
@@ -278,7 +279,7 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 	/* single polling interval query for items */
 	if (set.poller_id == 0) {
 		snprintf(query1, BUFSIZE,
-			"SELECT action, hostname, snmp_community, "
+			"SELECT SQL_NO_CACHE action, hostname, snmp_community, "
 				"snmp_version, snmp_username, snmp_password, "
 				"rrd_name, rrd_path, arg1, arg2, arg3, local_data_id, "
 				"rrd_num, snmp_port, snmp_timeout, "
@@ -290,7 +291,7 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 
 		/* host structure for uptime checks */
 		snprintf(query2, BIG_BUFSIZE,
-			"SELECT id, hostname, snmp_community, snmp_version, "
+			"SELECT SQL_NO_CACHE id, hostname, snmp_community, snmp_version, "
 				"snmp_username, snmp_password, snmp_auth_protocol, "
 				"snmp_priv_passphrase, snmp_priv_protocol, snmp_context, snmp_engine_id, snmp_port, snmp_timeout, max_oids, "
 				"availability_method, ping_method, ping_port, ping_timeout, ping_retries, "
@@ -305,7 +306,7 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 
 		/* data query structure for reindex detection */
 		snprintf(query4, BUFSIZE,
-			"SELECT data_query_id, action, op, assert_value, arg1"
+			"SELECT SQL_NO_CACHE data_query_id, action, op, assert_value, arg1"
 				" FROM poller_reindex"
 				" WHERE host_id = %i", host_id);
 
@@ -313,7 +314,7 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 		if (set.active_profiles != 1) {
 			if (set.total_snmp_ports == 1) {
 				snprintf(query5, BUFSIZE,
-					"SELECT action, hostname, snmp_community, "
+					"SELECT SQL_NO_CACHE action, hostname, snmp_community, "
 						"snmp_version, snmp_username, snmp_password, "
 						"rrd_name, rrd_path, arg1, arg2, arg3, local_data_id, "
 						"rrd_num, snmp_port, snmp_timeout, "
@@ -324,7 +325,7 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 					" %s", host_id, limits);
 			} else {
 				snprintf(query5, BUFSIZE,
-					"SELECT action, hostname, snmp_community, "
+					"SELECT SQL_NO_CACHE action, hostname, snmp_community, "
 						"snmp_version, snmp_username, snmp_password, "
 						"rrd_name, rrd_path, arg1, arg2, arg3, local_data_id, "
 						"rrd_num, snmp_port, snmp_timeout, "
@@ -337,7 +338,7 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 		} else {
 			if (set.total_snmp_ports == 1) {
 				snprintf(query5, BUFSIZE,
-					"SELECT action, hostname, snmp_community, "
+					"SELECT SQL_NO_CACHE action, hostname, snmp_community, "
 						"snmp_version, snmp_username, snmp_password, "
 						"rrd_name, rrd_path, arg1, arg2, arg3, local_data_id, "
 						"rrd_num, snmp_port, snmp_timeout, "
@@ -347,7 +348,7 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 					" %s", host_id, limits);
 			} else {
 				snprintf(query5, BUFSIZE,
-					"SELECT action, hostname, snmp_community, "
+					"SELECT SQL_NO_CACHE action, hostname, snmp_community, "
 						"snmp_version, snmp_username, snmp_password, "
 						"rrd_name, rrd_path, arg1, arg2, arg3, local_data_id, "
 						"rrd_num, snmp_port, snmp_timeout, "
@@ -380,7 +381,7 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 
 		/* number of agent's count for single polling interval */
 		snprintf(query9, BUFSIZE,
-			"SELECT snmp_port, count(snmp_port)"
+			"SELECT SQL_NO_CACHE snmp_port, count(snmp_port)"
 			" FROM poller_item"
 			" WHERE host_id = %i"
 			" GROUP BY snmp_port %s", host_id, limits);
@@ -388,21 +389,21 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 		/* number of agent's count for multiple polling intervals */
 		if (set.active_profiles != 1) {
 			snprintf(query10, BUFSIZE,
-				"SELECT snmp_port, count(snmp_port)"
+				"SELECT SQL_NO_CACHE snmp_port, count(snmp_port)"
 				" FROM poller_item"
 				" WHERE host_id = %i"
 				" AND rrd_next_step <= 0"
 				" GROUP BY snmp_port %s", host_id, limits);
 		} else {
 			snprintf(query10, BUFSIZE,
-				"SELECT snmp_port, count(snmp_port)"
+				"SELECT SQL_NO_CACHE snmp_port, count(snmp_port)"
 				" FROM poller_item"
 				" WHERE host_id = %i"
 				" GROUP BY snmp_port %s", host_id, limits);
 		}
 	} else {
 		snprintf(query1, BUFSIZE,
-			"SELECT action, hostname, snmp_community, "
+			"SELECT SQL_NO_CACHE action, hostname, snmp_community, "
 				"snmp_version, snmp_username, snmp_password, "
 				"rrd_name, rrd_path, arg1, arg2, arg3, local_data_id, "
 				"rrd_num, snmp_port, snmp_timeout, "
@@ -414,7 +415,7 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 
 		/* host structure for uptime checks */
 		snprintf(query2, BIG_BUFSIZE,
-			"SELECT id, hostname, snmp_community, snmp_version, "
+			"SELECT SQL_NO_CACHE id, hostname, snmp_community, snmp_version, "
 				"snmp_username, snmp_password, snmp_auth_protocol, "
 				"snmp_priv_passphrase, snmp_priv_protocol, snmp_context, snmp_engine_id, snmp_port, snmp_timeout, max_oids, "
 				"availability_method, ping_method, ping_port, ping_timeout, ping_retries, "
@@ -429,14 +430,14 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 
 		/* data query structure for reindex detection */
 		snprintf(query4, BUFSIZE,
-			"SELECT data_query_id, action, op, assert_value, arg1"
+			"SELECT SQL_NO_CACHE data_query_id, action, op, assert_value, arg1"
 				" FROM poller_reindex"
 				" WHERE host_id = %i", host_id);
 
 		/* multiple polling interval query for items */
 		if (set.active_profiles != 1) {
 			snprintf(query5, BUFSIZE,
-				"SELECT action, hostname, snmp_community, "
+				"SELECT SQL_NO_CACHE action, hostname, snmp_community, "
 					"snmp_version, snmp_username, snmp_password, "
 					"rrd_name, rrd_path, arg1, arg2, arg3, local_data_id, "
 					"rrd_num, snmp_port, snmp_timeout, "
@@ -448,7 +449,7 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 				" ORDER by snmp_port %s", host_id, set.poller_id, limits);
 		} else {
 			snprintf(query5, BUFSIZE,
-				"SELECT action, hostname, snmp_community, "
+				"SELECT SQL_NO_CACHE action, hostname, snmp_community, "
 					"snmp_version, snmp_username, snmp_password, "
 					"rrd_name, rrd_path, arg1, arg2, arg3, local_data_id, "
 					"rrd_num, snmp_port, snmp_timeout, "
@@ -477,7 +478,7 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 
 		/* number of agent's count for single polling interval */
 		snprintf(query9, BUFSIZE,
-			"SELECT snmp_port, count(snmp_port)"
+			"SELECT SQL_NO_CACHE snmp_port, count(snmp_port)"
 			" FROM poller_item"
 			" WHERE host_id = %i"
 			" AND poller_id = %i"
@@ -486,7 +487,7 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 		/* number of agent's count for multiple polling intervals */
 		if (set.active_profiles != 1) {
 			snprintf(query10, BUFSIZE,
-				"SELECT snmp_port, count(snmp_port)"
+				"SELECT SQL_NO_CACHE snmp_port, count(snmp_port)"
 				" FROM poller_item"
 				" WHERE host_id = %i"
 				" AND rrd_next_step <= 0"
@@ -494,7 +495,7 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 				" GROUP BY snmp_port %s", host_id, set.poller_id, limits);
 		} else {
 			snprintf(query10, BUFSIZE,
-				"SELECT snmp_port, count(snmp_port)"
+				"SELECT SQL_NO_CACHE snmp_port, count(snmp_port)"
 				" FROM poller_item"
 				" WHERE host_id = %i"
 				" AND poller_id = %i"
@@ -879,6 +880,9 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 
 									snprintf(poll_result, BUFSIZE, "%s", sysUptime);
 								} else if (strstr(reindex->arg1, ".1.3.6.1.2.1.1.3.0")) {
+								     // Ensure uptime is empty to start with
+								     snprintf(sysUptime, BUFSIZE, "");
+
 									// Check the legacy poll result first
 									poll_result = snmp_get(host, reindex->arg1);
 
@@ -899,9 +903,12 @@ void poll_host(int device_counter, int host_id, int host_thread, int host_thread
 
 									if (poll_result && is_numeric(poll_result)) {
 										snprintf(sysUptime, BUFSIZE, "%llu", atoll(poll_result) * 100);
-										snprintf(poll_result, BUFSIZE, "%s", sysUptime);
 									}
 
+									// Use the primed uptime to repopulate the poll_result
+									// This ensures whichever response was valid gets used
+									snprintf(poll_result, BUFSIZE, "%s", sysUptime);
+
 									if (is_debug_device(host->id)) {
 										SPINE_LOG(("Device[%i] HT[%i] DQ[%i] Extended Uptime Result: %s, Is Numeric: %d", host->id, host_thread, reindex->data_query_id, poll_result, is_numeric(poll_result) ));
 									} else {
@@ -2127,6 +2134,7 @@ char *exec_poll(host_t *current_host, char *command, int id, char *type) {
 	double begin_time = 0;
 	double end_time = 0;
 	double script_timeout;
+	double remaining_usec = 0;
 	struct timeval timeout;
 	char *proc_command;
 	char *result_string;
@@ -2232,10 +2240,16 @@ char *exec_poll(host_t *current_host, char *command, int id, char *type) {
 					end_time = get_time_as_double();
 
 					/* re-establish new timeout value */
-					timeout.tv_sec = rint(floor(script_timeout-(end_time-begin_time)));
-					timeout.tv_usec = rint((script_timeout-(end_time-begin_time)-timeout.tv_sec)*1000000);
+					timeout.tv_sec  = rint(floor(script_timeout-(end_time-begin_time)));
+					remaining_usec  = set.script_timeout - timeout.tv_sec - (end_time - begin_time);
 
-					if ((end_time - begin_time) < set.script_timeout) {
+					if (remaining_usec > 0) {
+						timeout.tv_usec = rint(remaining_usec * 1000000);
+					} else {
+						timeout.tv_usec = 0;
+					}
+
+					if (timeout.tv_sec + timeout.tv_usec > 0) {
 						goto retry;
 					} else {
 						SPINE_LOG(("WARNING: A script timed out while processing EINTR's."));
